@@ -2,26 +2,121 @@ import os
 import logging
 import numpy as np
 import scanpy as sc
-import pandas as pd
 import anndata as ad
+import pandas as pd
+from matplotlib.figure import Figure
 import matplotlib.pyplot as plt
 from scipy.sparse.csr import spmatrix
 from scipy.stats import chi2
-from typing import Sequence, Tuple, Iterable, Union
+from typing import Mapping, Sequence, Tuple, Iterable, Union
 from scipy.sparse import issparse
 from sklearn.metrics import adjusted_rand_score, normalized_mutual_info_score
 from sklearn.neighbors import NearestNeighbors
+from scETM.logging_utils import log_arguments
 import psutil
 
 _cpu_count: Union[None, int] = psutil.cpu_count(logical=False)
 if _cpu_count is None:
     _cpu_count: int = psutil.cpu_count(logical=True)
-logger = logging.getLogger(__name__)
+_logger = logging.getLogger(__name__)
+
+
+@log_arguments
+def evaluate(adata: ad.AnnData,
+    embedding_key: str = 'delta',
+    n_neighbors: int = 15,
+    resolutions: Iterable[float] = [0.01, 0.02, 0.04, 0.08, 0.16, 0.32, 0.64],
+    clustering_method: str = "leiden",
+    cell_type_col: str = "cell_types",
+    batch_col: Union[str, None] = "batch_indices",
+    draw: bool = True,
+    color_by: Iterable[str] = None,
+    return_fig: bool = False,
+    plot_fname: str = "umap",
+    plot_ftype: str = "pdf",
+    plot_dir: Union[str, None] = None,
+    min_dist: float = 0.3,
+    spread: float = 1,
+    n_jobs: int = -1,
+) -> Mapping[str, Union[float, None, Figure]]:
+    """Evaluate the clustering and batch correction performance of the given
+    embeddings, and optionally plot the embeddings.
+
+    Args:
+        adata: the dataset with the embedding to be evaluated.
+        embedding_key: the key to the embedding. Must be in adata.obsm.
+        n_neighbors: #neighbors used when computing neithborhood graph and
+            calculating entropy of batch mixing / kBET.
+        resolutions: a sequence of resolutions used for clustering.
+        clustering_method: clustering method used. Should be one of 'leiden' or
+            'louvain'.
+        cell_type_col: a key in adata.obs to the cell type column.
+        batch_col: a key in adata.obs to the batch column.
+        draw: whether to plot the embeddings.
+        return_fig: whether to return the Figure object. Useful for visualizing
+            the plot. Only used if draw is True.
+        color_by: a list of adata.obs column keys to color the embeddings by.
+            Only used if draw is True.
+        plot_fname: file name of the generated plot. Only used if draw is True.
+        plot_ftype: file type of the generated plot. Only used if draw is True.
+        plot_dir: directory to save the generated plot. If None, do not save
+            the plot. Only used if draw is True.
+        min_dist: the min_dist argument in sc.tl.umap. Only used if draw is
+            True.
+        spread: the spread argument in sc.tl.umap. Only used if draw is True.
+        n_jobs: # jobs to generate. If <= 0, this is set to the number of
+            physical cores.
+
+    Returns:
+        A dict storing the ARI with cell type, ebm and kBET of the given
+        embeddings.
+    """
+
+    if not pd.api.types.is_categorical_dtype(adata.obs[cell_type_col]):
+        _logger.warning("scETM.evaluate assumes discrete cell types. Converting cell_type_col to categorical.")
+    if not pd.api.types.is_categorical_dtype(adata.obs[batch_col]):
+        _logger.warning("scETM.evaluate assumes discrete batches. Converting batch_col to categorical.")
+
+    # calculate neighbors
+    sc.pp.neighbors(adata, n_neighbors=n_neighbors, use_rep=embedding_key)
+
+    # calculate clustering metrics
+    if cell_type_col in adata.obs:
+        cluster_key, best_ari = clustering(adata, resolutions=resolutions, cell_type_col=cell_type_col, batch_col=batch_col, clustering_method=clustering_method)
+    else:
+        cluster_key, best_ari = None, None
+
+    # calculate batch correction metrics
+    if batch_col and adata.obs[batch_col].nunique() > 1:
+        ebm = calculate_entropy_batch_mixing(adata, use_rep=embedding_key, batch_col=batch_col, n_neighbors=n_neighbors, calc_knn=False, n_jobs=n_jobs)
+        _logger.info(f'{embedding_key}_BE: {ebm:7.4f}')
+        k_bet = calculate_kbet(adata, use_rep=embedding_key, batch_col=batch_col, n_neighbors=n_neighbors, calc_knn=False, n_jobs=n_jobs)[2]
+        _logger.info(f'{embedding_key}_kBET: {k_bet:7.4f}')
+    else:
+        ebm = k_bet = None
+
+    # plot UMAP embeddings
+    if draw:
+        if color_by is None:
+            color_by = [cell_type_col] if batch_col is None else [batch_col, cell_type_col]
+        if cluster_key is not None:
+            color_by = [cluster_key] + color_by
+        fig = draw_embeddings(adata=adata, color_by=color_by, min_dist=min_dist, spread=spread,
+            ckpt_dir=plot_dir, fname=f'{plot_fname}.{plot_ftype}', return_fig=return_fig)
+    else:
+        fig = None
+    
+    return dict(
+        best_ari=best_ari,
+        ebm=ebm,
+        k_bet=k_bet,
+        fig = fig
+    )
 
 
 def _eff_n_jobs(n_jobs: int) -> int:
     """If n_jobs <= 0, set it as the number of physical cores _cpu_count"""
-    return n_jobs if n_jobs > 0 else _cpu_count
+    return int(n_jobs) if n_jobs > 0 else _cpu_count
 
 
 def calculate_nearest_neighbors(
@@ -74,7 +169,7 @@ def calculate_nearest_neighbors(
             method = "hnsw"
 
     if nsamples < n_neighbors:
-        logger.warning(f"Warning: in calculate_nearest_neighbors, number of samples = {nsamples} < n_neighbors = {n_neighbors}!\n Set n_neighbors to {nsamples}.")
+        _logger.warning(f"Warning: in calculate_nearest_neighbors, number of samples = {nsamples} < n_neighbors = {n_neighbors}!\n Set n_neighbors to {nsamples}.")
         n_neighbors = nsamples
 
     n_jobs = _eff_n_jobs(n_jobs)
@@ -121,16 +216,9 @@ def _calculate_kbet_for_one_chunk(knn_indices, attr_values, ideal_dist, n_neighb
     ns = knn_indices.shape[0]
     results = np.zeros((ns, 2))
     for i in range(ns):
-        observed_counts = (
-            pd.Series(attr_values[knn_indices[i, :]]).value_counts(sort=False).values
-        )
+        _, observed_counts = np.unique(attr_values[knn_indices[i, :]], return_counts = True)
         expected_counts = ideal_dist * n_neighbors
-        stat = np.sum(
-            np.divide(
-                np.square(np.subtract(observed_counts, expected_counts)),
-                expected_counts,
-            )
-        )
+        stat = np.sum((observed_counts - expected_counts) ** 2 / expected_counts)
         p_value = 1 - chi2.cdf(stat, dof)
         results[i, 0] = stat
         results[i, 1] = p_value
@@ -205,9 +293,10 @@ def calculate_kbet(
         accept_rate: kBET Acceptance rate of the sample.
     """
 
-    logger.info('Calculating kbet...')
+    _logger.info('Calculating kbet...')
     assert batch_col in adata.obs
-    assert adata.obs[batch_col].dtype.name == "category"
+    if adata.obs[batch_col].dtype.name != "category":
+        adata.obs[batch_col] = adata.obs[batch_col].astype('category')
 
     ideal_dist = (
         adata.obs[batch_col].value_counts(normalize=True, sort=False).values
@@ -300,7 +389,7 @@ def calculate_entropy_batch_mixing(
         score: the mean entropy of batch mixing, averaged from n_pools samples.
     """
 
-    logger.info('Calculating batch mixing entropy...')
+    _logger.info('Calculating batch mixing entropy...')
     nsample = adata.n_obs
 
     knn_indices = _get_knn_indices(adata, use_rep, n_neighbors, random_state, full_speed, n_jobs, calc_knn)
@@ -358,7 +447,7 @@ def clustering(
         clustering_func: function = sc.tl.louvain
     else:
         raise ValueError("Please specify louvain or leiden for the clustering method argument.")
-    logger.info(f'Performing {clustering_method} clustering')
+    _logger.info(f'Performing {clustering_method} clustering')
     assert cell_type_col in adata.obs, f"{cell_type_col} not in adata.obs"
     aris = []
     for res in resolutions:
@@ -370,9 +459,9 @@ def clustering(
         aris.append((res, ari, n_unique))
         if batch_col in adata.obs and adata.obs[batch_col].nunique() > 1:
             ari_batch = adjusted_rand_score(adata.obs[batch_col], adata.obs[col])
-            logger.info(f'Resolution: {res:5.3g}\tARI: {ari:7.4f}\tNMI: {nmi:7.4f}\tbARI: {ari_batch:7.4f}\t# labels: {n_unique}')
+            _logger.info(f'Resolution: {res:5.3g}\tARI: {ari:7.4f}\tNMI: {nmi:7.4f}\tbARI: {ari_batch:7.4f}\t# labels: {n_unique}')
         else:
-            logger.info(f'Resolution: {res:5.3g}\tARI: {ari:7.4f}\tNMI: {nmi:7.4f}\t# labels: {n_unique}')
+            _logger.info(f'Resolution: {res:5.3g}\tARI: {ari:7.4f}\tNMI: {nmi:7.4f}\t# labels: {n_unique}')
     
     aris.sort(key=lambda x: x[1], reverse=True)
     best_res, best_ari = aris[0][0], aris[0][1]
@@ -383,12 +472,11 @@ def draw_embeddings(adata: ad.AnnData,
         color_by: Union[str, Sequence[str], None] = None,
         min_dist: float = 0.3,
         spread: float = 1,
-        save: bool = True,
         ckpt_dir: str = '.',
         fname: str = "umap.pdf",
-        show: bool = False,
-        dpi: int = 300,
-    ) -> None:
+        return_fig: bool = False,
+        dpi: int = 300
+    ) -> Union[None, Figure]:
     """Embed, plot and optionally save the neighborhood graph with UMAP.
 
     Requires having run sc.pp.neighbors first.
@@ -407,21 +495,26 @@ def draw_embeddings(adata: ad.AnnData,
         spread: The effective scale of embedded points. In combination with
             `min_dist` this determines how clustered/clumped the embedded
             points are.
-        save: whether to save the plot to ckpt_dir.
-        ckpt_dir: where to save the plot. Only used if save = True.
-        fname: file name of the saved plot. Only used if save = True.
-        show: whether to show the plot (plt.show()). Useful for visualizing
-            the plot in an interactive environment.
-        dpi: the dpi of the saved plot. Only used if save = True.
+        ckpt_dir: where to save the plot. If None, do not save the plot.
+        fname: file name of the saved plot. Only used if ckpt_dir is not None.
+        return_fig: whether to return the Figure object. Useful for visualizing
+            the plot.
+        dpi: the dpi of the saved plot. Only used if ckpt_dir is not None.
+
+    Returns:
+        If return_fig is True, return the figure containing the plot.
     """
 
     sc.tl.umap(adata, min_dist=min_dist, spread=spread)
-    fig = sc.pl.umap(adata, color=color_by, show=show, return_fig=True)
-    if save:
+    fig = sc.pl.umap(adata, color=color_by, show=False, return_fig=True)
+    if ckpt_dir is not None:
+        assert os.path.exists(ckpt_dir), f'ckpt_dir {ckpt_dir} does not exist.'
         fig.savefig(
             os.path.join(ckpt_dir, fname),
             dpi=dpi, bbox_inches='tight'
         )
+    if return_fig:
+        return fig
     fig.clf()
     plt.close(fig)
 
