@@ -16,7 +16,7 @@ from .model_utils import (
     get_kl
 )
 
-logger = logging.getLogger(__name__)
+_logger = logging.getLogger(__name__)
 
 class scETM(BaseCellModel):
     """Single-cell Embedded Topic Model.
@@ -180,7 +180,7 @@ class scETM(BaseCellModel):
         """Initialize batch and global biases given the constant attributes."""
 
         if self.n_batches <= 1:
-            logger.warning(f'n_batches == {self.n_batches}, disabling batch bias')
+            _logger.warning(f'n_batches == {self.n_batches}, disabling batch bias')
             self.enable_batch_bias = False
 
         if self.enable_batch_bias:
