@@ -3,24 +3,23 @@ import copy
 from pathlib import Path
 import pandas as pd
 import time
-from typing import Iterable, List, Mapping, Sequence, Union, Tuple
+from typing import Sequence, Union, Tuple
 import psutil
 import logging
 from collections import defaultdict
 
 import numpy as np
 import anndata
-import scanpy as sc
 import torch
 from torch import nn
 from torch import optim
 
 from scETM.batch_sampler import CellSampler, MultithreadedCellSampler
-from scETM.eval_utils import calculate_kbet, clustering, draw_embeddings, calculate_entropy_batch_mixing
+from scETM.eval_utils import evaluate
 from scETM.model import BaseCellModel, scETM
 from scETM.logging_utils import initialize_logger, log_arguments
 
-logger = logging.getLogger(__name__)
+_logger = logging.getLogger(__name__)
 
 
 class UnsupervisedTrainer:
@@ -54,7 +53,7 @@ class UnsupervisedTrainer:
     def __init__(self,
         model: BaseCellModel,
         adata: anndata.AnnData,
-        ckpt_dir: str = ".",
+        ckpt_dir: Union[str, None] = None,
         test_ratio: float = 0.,
         data_split_seed: int = 1,
         init_lr: float = 5e-3,
@@ -107,11 +106,13 @@ class UnsupervisedTrainer:
         if restore_epoch > 0:
             self.ckpt_dir = ckpt_dir
             self.load_ckpt(restore_epoch, self.ckpt_dir)
-        else:
+        elif ckpt_dir is not None:
             self.ckpt_dir = os.path.join(ckpt_dir, f"{self.train_instance_name}_{time.strftime('%m_%d-%H_%M_%S')}")
             os.makedirs(self.ckpt_dir, exist_ok=True)
             initialize_logger(self.ckpt_dir)
-        logger.info(f'ckpt_dir: {self.ckpt_dir}')
+            _logger.info(f'ckpt_dir: {self.ckpt_dir}')
+        else:
+            self.ckpt_dir = None
 
     @log_arguments
     def load_ckpt(self, restore_epoch: int, ckpt_dir: Union[str, None] = None) -> None:
@@ -128,14 +129,15 @@ class UnsupervisedTrainer:
 
         if ckpt_dir is None:
             ckpt_dir = self.ckpt_dir
-        self.model.load_state_dict(torch.load(os.path.join(
-            ckpt_dir, f'model-{restore_epoch}')))
-        logger.info('Parameters restored.')
-        self.optimizer.load_state_dict(torch.load(os.path.join(
-            ckpt_dir, f'opt-{restore_epoch}')))
-        logger.info('Optimizer restored.')
+        assert ckpt_dir is not None and os.path.exists(ckpt_dir), f"ckpt_dir {ckpt_dir} does not exist."
+        model_ckpt_path = os.path.join(ckpt_dir, f'model-{restore_epoch}')
+        self.model.load_state_dict(torch.load(model_ckpt_path))
+        _logger.info(f'Parameters restored from {model_ckpt_path}.')
+        optim_ckpt_path = os.path.join(ckpt_dir, f'opt-{restore_epoch}')
+        self.optimizer.load_state_dict(torch.load(optim_ckpt_path))
+        _logger.info(f'Optimizer restored from {optim_ckpt_path}.')
         initialize_logger(self.ckpt_dir)
-        logger.info(f'ckpt_dir: {self.ckpt_dir}')
+        _logger.info(f'ckpt_dir: {self.ckpt_dir}')
         self.update_step(restore_epoch * self.steps_per_epoch)
 
     @staticmethod
@@ -150,7 +152,7 @@ class UnsupervisedTrainer:
         torch.backends.cudnn.deterministic = True
         torch.backends.cudnn.benchmark = False
         np.random.seed(seed)
-        logger.info(f'Set seed to {seed}.')
+        _logger.info(f'Set seed to {seed}.')
 
     @staticmethod
     def _get_kl_weight(
@@ -270,14 +272,14 @@ class UnsupervisedTrainer:
 
             # log and evaluate
             if self.epoch >= next_ckpt_epoch or self.epoch >= n_epochs:
-                logger.info('=' * 10 + f'Epoch {self.epoch:.0f}' + '=' * 10)
+                _logger.info('=' * 10 + f'Epoch {self.epoch:.0f}' + '=' * 10)
 
                 # log memory cost
-                logger.info(repr(psutil.Process().memory_info()))
+                _logger.info(repr(psutil.Process().memory_info()))
                 # log current lr and kl_weight
                 if self.lr_decay:
-                    logger.info(f'{"lr":12s}: {self.lr}')
-                logger.info(f'{"kl_weight":12s}: {self._get_kl_weight(self.epoch, n_epochs):12.4f}')
+                    _logger.info(f'{"lr":12s}: {self.lr}')
+                _logger.info(f'{"kl_weight":12s}: {self._get_kl_weight(self.epoch, n_epochs):12.4f}')
 
                 # log statistics of tracked items
                 recorder.log_and_clear_record()
@@ -287,7 +289,9 @@ class UnsupervisedTrainer:
                     current_eval_kwargs['plot_fname'] = current_eval_kwargs['plot_fname'] + f'_epoch{int(self.epoch)}'
                     if self.test_adata is not self.adata:
                         test_nll = self.model.get_embeddings_and_nll(self.test_adata, self.batch_size, batch_col=batch_col)
-                        logger.info(f'test nll: {test_nll:7.4f}')
+                        _logger.info(f'test nll: {test_nll:7.4f}')
+                    else:
+                        test_nll = None
                     self.model.get_embeddings_and_nll(self.adata, self.batch_size, batch_col=batch_col)
                     result = evaluate(adata = self.adata, **current_eval_kwargs)
                     if eval_result_log_path is not None:
@@ -295,101 +299,26 @@ class UnsupervisedTrainer:
                             # ckpt_dir, epoch, test_nll, best_ari, k_bet, ebm, time, seed
                             f.write(f'{Path(self.ckpt_dir).name}\t'
                                     f'{self.epoch}\t'
-                                    f'{result["test_nll"]}\t'
+                                    f'{test_nll}\t'
                                     f'{result["best_ari"]}\t'
                                     f'{result["k_bet"]}\t'
                                     f'{result["ebm"]}\t'
                                     f'{time.strftime("%m_%d-%H_%M_%S")}\t'
                                     f'{self.seed}\n')
 
-                if next_ckpt_epoch and save_model_ckpt:
+                if next_ckpt_epoch and save_model_ckpt and self.ckpt_dir is not None:
                     # checkpointing
                     torch.save(self.model.state_dict(), os.path.join(self.ckpt_dir, f'model-{next_ckpt_epoch}'))
                     torch.save(self.optimizer.state_dict(), os.path.join(self.ckpt_dir, f'opt-{next_ckpt_epoch}'))
 
-                logger.info('=' * 10 + f'End of evaluation' + '=' * 10)
+                _logger.info('=' * 10 + f'End of evaluation' + '=' * 10)
                 next_ckpt_epoch += eval_every
 
         del recorder
-        logger.info("Optimization Finished: %s" % self.ckpt_dir)
+        _logger.info("Optimization Finished: %s" % self.ckpt_dir)
         if isinstance(sampler, MultithreadedCellSampler):
             sampler.join(0.1)
 
-@log_arguments
-def evaluate(adata: anndata.AnnData,
-    embedding_key: str = 'delta',
-    n_neighbors: int = 15,
-    resolutions: Iterable[float] = [0.01, 0.02, 0.04, 0.08, 0.16, 0.32, 0.64],
-    clustering_method: str = "leiden",
-    cell_type_col: str = "cell_types",
-    batch_col: Union[str, None] = "batch_indices",
-    draw: bool = True,
-    color_by: Iterable[str] = None,
-    plot_fname: str = "umap",
-    plot_ftype: str = "pdf",
-    plot_dir: str = ".",
-    min_dist: float = 0.3,
-    spread: float = 1,
-) -> Mapping[str, float]:
-    """Evaluate the clustering and batch correction performance of the given
-    embeddings, and optionally plot the embeddings.
-
-    Args:
-        adata: the dataset with the embedding to be evaluated.
-        embedding_key: the key to the embedding. Must be in adata.obsm.
-        n_neighbors: #neighbors used when computing neithborhood graph and
-            calculating entropy of batch mixing / kBET.
-        resolutions: a sequence of resolutions used for clustering.
-        clustering_method: clustering method used. Should be one of 'leiden' or
-            'louvain'.
-        cell_type_col: a key in adata.obs to the cell type column.
-        batch_col: a key in adata.obs to the batch column.
-        draw: whether to save the embeddings plots.
-        color_by: a list of adata.obs column keys to color the embeddings by.
-        plot_fname: file name of the generated plot.
-        plot_ftype: file type of the generated plot.
-        plot_dir: directory to save the generated plot.
-        min_dist: the min_dist argument in sc.tl.umap.
-        spread: the spread argument in sc.tl.umap.
-
-    Returns:
-        A dict storing the ARI with cell type, ebm and kBET of the given
-        embeddings.
-    """
-
-    # calculate neighbors
-    sc.pp.neighbors(adata, n_neighbors=n_neighbors, use_rep=embedding_key)
-
-    # calculate clustering metrics
-    if cell_type_col in adata.obs:
-        cluster_key, best_ari = clustering(adata, resolutions=resolutions, cell_type_col=cell_type_col, batch_col=batch_col, clustering_method=clustering_method)
-    else:
-        cluster_key, best_ari = None, None
-
-    # calculate batch correction metrics
-    if batch_col and adata.obs[batch_col].nunique() > 1:
-        ebm = calculate_entropy_batch_mixing(adata, use_rep=embedding_key, batch_col=batch_col, n_neighbors=n_neighbors, calc_knn=False)
-        logger.info(f'{embedding_key}_BE: {ebm:7.4f}')
-        k_bet = calculate_kbet(adata, use_rep=embedding_key, batch_col=batch_col, n_neighbors=n_neighbors, calc_knn=False)[2]
-        logger.info(f'{embedding_key}_kBET: {k_bet:7.4f}')
-    else:
-        ebm = k_bet = None
-
-    # plot UMAP embeddings
-    if draw:
-        if color_by is None:
-            color_by = [cell_type_col] if batch_col is None else [batch_col, cell_type_col]
-        if cluster_key is not None:
-            color_by = [cluster_key] + color_by
-        draw_embeddings(adata=adata, color_by=color_by, min_dist=min_dist, spread=spread,
-            ckpt_dir=plot_dir, fname=f'{plot_fname}.{plot_ftype}')
-    
-    return dict(
-        best_ari=best_ari,
-        ebm=ebm,
-        k_bet=k_bet
-    )
-
 
 class _stats_recorder:
     def __init__(self, record_log_path: Union[str, None] = None, fmt: str = "12.4f") -> None:
@@ -414,7 +343,7 @@ class _stats_recorder:
 
     def log_and_clear_record(self):
         for key, val in self.record.items():
-            logger.info(f'{key:12s}: {np.mean(val):{self.fmt}}')
+            _logger.info(f'{key:12s}: {np.mean(val):{self.fmt}}')
         self.record = defaultdict(list)
 
     def __del__(self):
@@ -443,7 +372,7 @@ def train_test_split(
     train_indices = list(set(range(adata.n_obs)).difference(test_indices))
     train_adata = adata[adata.obs_names[train_indices], :]
     test_adata = adata[adata.obs_names[test_indices], :]
-    logger.info(f'Keeping {test_adata.n_obs} cells ({test_ratio:g}) as test data.')
+    _logger.info(f'Keeping {test_adata.n_obs} cells ({test_ratio:g}) as test data.')
     return train_adata, test_adata
 
 
